import { Component, Input } from "@angular/core";

import { Result, cLog, Message, MessageCode, MessageSeverity } from "jalhyd";

import { CalculatorResults } from "../../results/calculator-results";
import { PabResults } from "../../results/pab-results";
import { I18nService } from "../../services/internationalisation.service";
import { ResultsComponentDirective } from "../fixedvar-results/results.component";

@Component({
    selector: "pab-results",
    templateUrl: "./pab-results.component.html",
    styleUrls: [
        "./pab-results.component.scss"
    ]
})
export class PabResultsComponent extends ResultsComponentDirective {

    /** résultats non mis en forme */
    private _pabResults: PabResults;

    constructor(
        private i18nService: I18nService,
    ) {
        super();
    }

    @Input()
    public set results(rs: CalculatorResults[]) {
        this._pabResults = undefined;
        if (rs.length > 0 && rs[0] instanceof PabResults) {
            this._pabResults = rs[0] as PabResults;
        }
    }

    private mergeGlobalLog(result: Result, log: cLog) {
        if (result) {
            if (result.hasGlobalLog()) {
                log.addLog(result.globalLog);
            }
            // if no parameter is varying, 1st element log is considered "global"
            if (this.pabResults.variatedParameters.length === 0) {
                if (result.hasResultElements() && result.resultElement.hasLog()) {
                    log.addLog(result.log);
                }
            }
        }
    }

    /**
     * Returns the number of errors, warnings, infos among children logs
     */
    private logStats(): any {
        const ret = {
            info: 0,
            warning: 0,
            error: 0
        };
        if (this._pabResults.result && this._pabResults.result.hasLog()) {
            for (const re of this._pabResults.result.resultElements) {
                if (re.hasLog()) {
                    for (const m of re.log.messages) {
                        const s = m.getSeverity();
                        switch (s) {
                            case MessageSeverity.INFO:
                                ret.info ++;
                                break;
                            case MessageSeverity.WARNING:
                                ret.warning ++;
                                break;
                            case MessageSeverity.ERROR:
                                ret.error ++;
                                break;
                        }
                    }
                }
            }
        }
        for (const cr of this._pabResults.cloisonsResults) {
            if (cr && cr.hasLog()) {
                for (const re of cr.resultElements) {
                    if (re.hasLog()) {
                        for (const m of re.log.messages) {
                            const s = m.getSeverity();
                            switch (s) {
                                case MessageSeverity.INFO:
                                    ret.info ++;
                                    break;
                                case MessageSeverity.WARNING:
                                    ret.warning ++;
                                    break;
                                case MessageSeverity.ERROR:
                                    ret.error ++;
                                    break;
                            }
                        }
                    }
                }
            }
        }
        if (this._pabResults.cloisonAvalResults && this._pabResults.cloisonAvalResults.hasLog()) {
            for (const re of this._pabResults.cloisonAvalResults.resultElements) {
                if (re.hasLog()) {
                    for (const m of re.log.messages) {
                        const s = m.getSeverity();
                        switch (s) {
                            case MessageSeverity.INFO:
                                ret.info ++;
                                break;
                            case MessageSeverity.WARNING:
                                ret.warning ++;
                                break;
                            case MessageSeverity.ERROR:
                                ret.error ++;
                                break;
                        }
                    }
                }
            }
        }
        return ret;
    }

    /*
     * Retourne les logs à afficher dans le composant de log global, au dessus
     * du sélecteur d'itération : messages globaux et / ou résumé des messages
     * spécifiques à chaque ResultElement
     */
    public get globalLog(): cLog {
        const l = new cLog();
        if (this._pabResults && this.pabResults.variatedParameters.length > 0) {
            this.mergeGlobalLog(this._pabResults.result, l);
            // un problème avec la PAB en général / les cloisons, à une étape quelconque ?
            if (
                (this.pabResults.hasLog)
                && l.messages.length === 0 // existing global messages make generic message below useless
            ) {
                const logStats = this.logStats();
                const m = new Message(MessageCode.WARNING_PROBLEMS_ENCOUNTERED);
                m.extraVar.info = "" + logStats.info; // to avoid displaying fixed number of digits
                m.extraVar.warning = "" + logStats.warning;
                m.extraVar.error = "" + logStats.error;
                l.add(m);
                // l.add(new Message(MessageCode.WARNING_PROBLEMS_ENCOUNTERED));
            }
        } // sinon pas de log global (aucun paramètre ne varie)
        return l;
    }

    /**
     * Retourne les logs à afficher dans le composant de log local, en dessous
     * du sélecteur d'itération : messages concernant l'itération (le ResultElement)
     * en cours
     */
    public get iterationLog(): cLog {
        const l = new cLog();
        if (this._pabResults) {
            if (this.pabResults.variatedParameters.length > 0) {
                // A. si un paramètre varie
                const vi = this._pabResults.variableIndex;
                // log de la PAB pour l'itération en cours
                if (
                    this._pabResults.result
                    && this._pabResults.result.hasResultElements()
                    && this._pabResults.result.resultElements[vi]
                    && this._pabResults.result.resultElements[vi].hasLog()
                ) {
                    l.addLog(this._pabResults.result.resultElements[vi].log);
                }
                // logs des enfants pour l'itération en cours
                for (const cr of this._pabResults.cloisonsResults) {
                    if (cr && cr.hasResultElements() && cr.resultElements[vi].hasLog()) {
                        l.addLog(cr.resultElements[vi].log);
                    }
                }
                if (this._pabResults.cloisonAvalResults && this._pabResults.cloisonAvalResults.resultElements[vi].hasLog()) {
                    l.addLog(this._pabResults.cloisonAvalResults.resultElements[vi].log);
                }
            } else {
                // B. si aucun paramètre ne varie
                this.mergeGlobalLog(this._pabResults.result, l); // faut bien mettre le log global quelque part
                // logs des enfants
                for (const cr of this._pabResults.cloisonsResults) {
                    if (cr && cr.hasResultElements() && cr.resultElement.hasLog()) {
                        l.addLog(cr.resultElement.log);
                    }
                }
                if (this._pabResults.cloisonAvalResults && this._pabResults.cloisonAvalResults.resultElement.hasLog()) {
                    l.addLog(this._pabResults.cloisonAvalResults.resultElement.log);
                }
            }
        }
        return l;
    }

    public get pabResults() {
        return this._pabResults;
    }

    public get hasResults(): boolean {
        return this._pabResults && this._pabResults.hasResults;
    }

    public get hasDisplayableResults(): boolean {
        let ret = this._pabResults && this._pabResults.hasResults;
        if (
            this._pabResults
            && this._pabResults.variatedParameters
            && this._pabResults.variatedParameters.length > 0
        ) {
            ret = ret
            && this._pabResults.variableIndex !== undefined
            && this._pabResults.result.resultElements[this._pabResults.variableIndex] !== undefined
            && this._pabResults.result.resultElements[this._pabResults.variableIndex].ok;
        }
        return ret;
    }

    public get uitextGeneralLogTitle(): string {
        return this.i18nService.localizeText("INFO_TITREJOURNAL_GLOBAL");
    }

}
