import { Component, ViewChild, Input, OnChanges, AfterViewInit, ElementRef } from "@angular/core";

import {
    acSection, cSnTrapez, ParamsSectionTrapez, cSnRectang, ParamsSectionRectang, cSnCirc,
    ParamsSectionCirc, cSnPuiss, ParamsSectionPuiss, Result
} from "jalhyd";

import { ResultsComponentDirective } from "../fixedvar-results/results.component";

@Component({
    selector: "section-canvas",
    templateUrl: "./section-canvas.component.html",
    styleUrls: [
        "./section-canvas.component.scss"
    ]
})
export class SectionCanvasComponent extends ResultsComponentDirective implements AfterViewInit, OnChanges {

    private static labelColors: { [key: string]: any; } = {
        "Hs": { r: 255, g: 0, b: 0 },
        "Hsc": { r: 0, g: 0, b: 255 },
        "Yn": { r: 16, g: 128, b: 16 },
        "Ycor": { r: 128, g: 128, b: 128 },
        "Yc": { r: 255, g: 128, b: 0 },
        "Ycon": { r: 255, g: 0, b: 255 },
        "Y": { r: 50, g: 50, b: 50 }
    };

    /** marges gauche/droite pour le texte (pixels) */
    private _textMargin = 90;

    /** marge basse (pixels) */
    private _bottomMargin = 5;

    /** facteurs d'échelle (coordonnées en m <-> coordonnées en pixels) */
    private _scaleX: number;
    private _scaleY: number;

    private _section: acSection;

    private _result: Result;

    private _size: number;

    // tirants
    private _levels: Object[] = [];

    public get width(): number {
        // return this._calcCanvas.nativeElement.width;
        return this._size;
    }

    public get height(): number {
        // return this._calcCanvas.nativeElement.height;
        return this._size;
    }

    private _context2d: CanvasRenderingContext2D;

    @ViewChild("canvas", { static: true })
    private _calcCanvas: ElementRef;

    @Input()
    public set section(s: acSection) {
        this._section = s;
    }

    @Input()
    public set result(r: Result) {
        this._result = r;
    }

    @Input()
    public set size(s: number) {
        this._size = s;
    }

    // redessine le canvas chaque fois qu'une entrée change
    public ngOnChanges() {
        setTimeout(() => {
            this.draw();
        }, 10);
    }

    public ngAfterViewInit() { // wait for the view to init before using the element
        this._context2d = this._calcCanvas.nativeElement.getContext("2d");
        this.draw();
    }

    public reset() {
        this._levels = [];
        this.clear();
    }

    public addLevel(val: number, label: string, rgb: {}) {
        this._levels.push({ val, label, rgb });
    }

    // max des niveaux à représenter
    private getMaxLevel(): number {
        // max de la cote de berge et des niveaux (qui sont triés)
        return Math.max(this._section.prms.YB.v, this._levels[this._levels.length - 1]["val"]);
    }

    private isSectionLevel(s: string) {
        for (const k in SectionCanvasComponent.labelColors) {
            if (k === s) {
                return true;
            }
        }
        return false;
    }

    public draw() {
        // console.log(">> redrawing at size", this._size);
        if (this._context2d && this._section) {
            this.reset();

            // traduction des symboles des variables calculées
            const re = this._result.resultElement;
            if (re !== undefined) {
                for (const k in re.values) {
                    if (k !== re.vCalcSymbol) {
                        const lbl = k.toUpperCase();
                        const er = re.getValue(k);
                        // this._resultElement.addExtraResult(lbl, er);

                        if (this.isSectionLevel(k)) {
                            this.addLevel(er, k + " = " + this.formattedValue(er), SectionCanvasComponent.labelColors[k]);
                        }
                    }
                }
            }

            // ajout du tirant d'eau saisi
            const valY = this._result.sourceNub.getParameter("Y").singleValue;
            this.addLevel(valY, "Y = " + this.formattedValue(valY), SectionCanvasComponent.labelColors["Y"]);

            this.sortLevels();
            this.drawFrame();
            const maxWidth = this.drawSection();
            this.drawLevels(maxWidth);
        }
    }

    /**
     * dessin des pointillés au dessus de la berge
     * xmin,xmax : position gauche/droite
     * yb : cote de berge
     * maxHeight : valeur maxi des cotes
     */
    private drawTopDashLines(xmin: number, xmax: number, yb: number, maxHeight: number) {
        this.setLineWidth(2);
        this.setLineDash([5]);
        this.setStrokeColor(128, 128, 128);
        this.drawSectionLine(xmin, yb, xmin, maxHeight);
        this.drawSectionLine(xmax, yb, xmax, maxHeight);
    }

    private drawSectionTrapez(): number {
        const sect: cSnTrapez = <cSnTrapez>this._section;
        const prms: ParamsSectionTrapez = <ParamsSectionTrapez>sect.prms;

        // cote de berge
        const yb: number = prms.YB.v;

        // largeur de la partie pentue
        const lp: number = prms.Fruit.v * prms.YB.v;

        // largeur totale de la section
        const maxWidth: number = lp * 2 + prms.LargeurFond.v;

        // hauteur totale de la section
        let maxHeight: number = this.getMaxLevel();
        maxHeight *= 1.1;

        this.computeScale(maxWidth, maxHeight);

        // dessin de la section

        this.setStrokeColor(0, 0, 0);
        this.setLineWidth(5);
        this.drawSectionLine(0, yb, lp, 0);
        this.drawSectionLine(lp, 0, lp + prms.LargeurFond.v, 0);
        this.drawSectionLine(lp + prms.LargeurFond.v, 0, maxWidth, yb);

        // pointillés du haut

        this.drawTopDashLines(0, maxWidth, yb, maxHeight);

        return maxWidth;
    }

    private drawSectionRect() {
        const sect: cSnRectang = <cSnRectang>this._section;
        const prms: ParamsSectionRectang = <ParamsSectionRectang>sect.prms;

        // cote de berge
        const yb: number = prms.YB.v;

        // largeur totale de la section
        const maxWidth: number = prms.LargeurBerge.v;

        // hauteur totale de la section
        let maxHeight: number = this.getMaxLevel();
        maxHeight *= 1.1;

        this.computeScale(maxWidth, maxHeight);

        // dessin de la section

        this.setStrokeColor(0, 0, 0);
        this.setLineWidth(5);
        this.drawSectionLine(0, yb, 0, 0);
        this.drawSectionLine(0, 0, prms.LargeurBerge.v, 0);
        this.drawSectionLine(prms.LargeurBerge.v, 0, maxWidth, yb);

        // pointillés du haut

        this.drawTopDashLines(0, maxWidth, yb, maxHeight);

        return maxWidth;
    }

    private drawSectionCirc() {
        const sect: cSnCirc = <cSnCirc>this._section;
        const prms: ParamsSectionCirc = <ParamsSectionCirc>sect.prms;

        // cote de berge
        const yb: number = prms.YB.v;

        // diamètre, rayon
        const D: number = prms.D.v;
        const r: number = D / 2;

        try {
            // largeur au miroir
            const B: Result = sect.CalcSection("B", yb);
            if (!B.ok) {
                throw B;
            }

            // largeur totale de la section
            const maxWidth: number = yb < r ? B.vCalc : D;

            // hauteur totale de la section
            let maxHeight: number = this.getMaxLevel();
            maxHeight *= 1.1;

            this.computeScale(maxWidth, maxHeight);

            // dessin de la section

            this.setStrokeColor(0, 0, 0);
            this.setLineWidth(5);

            const wx: number = maxWidth / 2;
            const alpha: Result = sect.CalcSection("Alpha", yb);
            if (!alpha.ok) {
                throw alpha;
            }

            const s: number = Math.PI / 2 - alpha.vCalc;
            const e: number = Math.PI / 2 + alpha.vCalc;
            this.drawSectionEllipse(wx, r, r, r, 0, s, e);

            // pointillés du haut

            const w: number = yb > r ? (D - B.vCalc) / 2 : 0;
            this.drawTopDashLines(w, maxWidth - w, yb, maxHeight);

            return maxWidth;
        } catch (e) {
            const res: Result = e as Result;
            this.drawText("error : " + res.log.toString(), 0, 0);
        }
    }

    private drawSectionPara() {
        const sect: cSnPuiss = <cSnPuiss>this._section;
        const prms: ParamsSectionPuiss = <ParamsSectionPuiss>sect.prms;

        // cote de berge
        const yb: number = prms.YB.v;

        // largeur au miroir
        const B: number = prms.LargeurBerge.v;

        // largeur totale de la section
        const maxWidth: number = B;

        // hauteur totale de la section
        let maxHeight: number = this.getMaxLevel();
        maxHeight *= 1.1;

        this.computeScale(maxWidth, maxHeight);

        // contour de la section

        this.setStrokeColor(0, 0, 0);
        this.setLineWidth(5);
        const k: number = prms.k.v;
        const lambda: number = B / Math.pow(yb, k);
        const inv_k: number = 1 / k;

        const n = 20;
        this._context2d.beginPath();
        for (let x: number = -B / 2; x <= B / 2; x += B / n) {
            const y: number = Math.pow(Math.abs(x) * 2 / lambda, inv_k);
            this._context2d.lineTo(this.Xm2pix(x + B / 2), this.Ym2pix(y));
        }
        this._context2d.stroke();

        // pointillés du haut
        this.drawTopDashLines(0, maxWidth, yb, maxHeight);

        return maxWidth;
    }

    private drawSectionEllipse(x: number, y: number, rX: number, rY: number, rot: number, start: number, end: number) {
        this.drawEllipse(this.Xm2pix(x), this.Ym2pix(y), rX * this._scaleX, rY * this._scaleY, rot, start, end);
    }

    private drawText(s: string, x: number, y: number, align?: string) {
        this.fillText(s, this.Xm2pix(x), this.Ym2pix(y), align);
    }

    private drawSectionLine(x1: number, y1: number, x2: number, y2: number) {
        this.drawLine(this.Xm2pix(x1), this.Ym2pix(y1), this.Xm2pix(x2), this.Ym2pix(y2));
    }

    /**
     * dessin de la section
     * @returns largeur de la section (en m)
     */
    private drawSection(): number {
        if (this._section instanceof cSnTrapez) {
            return this.drawSectionTrapez();
        }
        if (this._section instanceof cSnRectang) {
            return this.drawSectionRect();
        }
        if (this._section instanceof cSnCirc) {
            return this.drawSectionCirc();
        }
        if (this._section instanceof cSnPuiss) {
            return this.drawSectionPara();
        }
        throw new Error("SectionCanvasComponent.drawSection() : type de section non pris en charge");
    }

    private computeScale(maxWidth: number, maxHeight: number) {
        this._scaleX = (this._size - 2 * this._textMargin) / maxWidth;
        this._scaleY = (this._size - this._bottomMargin) / maxHeight;
    }

    /**
     * convertit une abscisse en m en pixels
     */
    private Xm2pix(x: number) {
        return this._textMargin + x * this._scaleX;
    }

    /**
     * convertit une ordonnée en m en pixels
     */
    private Ym2pix(y: number) {
        return this._size - this._bottomMargin - y * this._scaleY;
    }

    private sortLevels() {
        this._levels.sort((a, b) => {
            if (a["val"] < b["val"]) {
                return -1;
            }
            if (a["val"] > b["val"]) {
                return 1;
            }
            return 0;
        });
    }

    private drawLevels(maxWidth: number) {
        let left = true;

        this.resetLineDash();
        this.setLineWidth(1);
        this.setFont("12px sans- serif");
        for (const l of this._levels) {
            const y = l["val"];
            const col = l["rgb"];
            this.setStrokeColor(col["r"], col["g"], col["b"]);
            this.drawSectionLine(0, y, maxWidth, y);

            this.setFillColor(col["r"], col["g"], col["b"]);
            if (left) {
                this.drawText(l["label"], -0.1, y, "right");
            } else {
                this.drawText(l["label"], maxWidth + 0.1, y, "left");
            }
            left = !left;
        }
    }

    // contour du canvas
    private drawFrame() {
        this.clear();
        this.resetLineDash();
        this.setStrokeColor(128, 128, 128);
        this.drawRect(0, 0, this.width, this.height);
    }

    public clear() {
        if (this._context2d) {
            this._context2d.clearRect(0, 0, this.width, this.height);
        }
    }

    public setStrokeColor(r: number, g: number, b: number) {
        const col: string = "rgb(" + r + "," + g + "," + b + ")";
        this._context2d.strokeStyle = col;
    }

    public setFillColor(r: number, g: number, b: number) {
        const col: string = "rgb(" + r + "," + g + "," + b + ")";
        this._context2d.fillStyle = col;
    }

    public setFont(f: string) {
        this._context2d.font = f;
    }

    public fillText(s: string, x: number, y: number, align?: any) {
        if (align) {
            this._context2d.textAlign = align;
        }
        this._context2d.fillText(s, x, y);
    }

    public setLineWidth(w: number) {
        this._context2d.lineWidth = w;
    }

    public setLineDash(d: number[]) {
        this._context2d.setLineDash(d);
    }

    public resetLineDash() {
        this._context2d.setLineDash([]);
    }

    public drawRect(x1: number, y1: number, w: number, h: number) {
        this._context2d.strokeRect(x1, y1, w, h);
    }

    public drawLine(x1: number, y1: number, x2: number, y2: number) {
        this._context2d.beginPath();
        this._context2d.moveTo(x1, y1);
        this._context2d.lineTo(x2, y2);
        this._context2d.stroke();
    }

    /**
     *
     * @param x The x axis of the coordinate for the ellipse's center.
     * @param y The y axis of the coordinate for the ellipse's center.
     * @param radiusX The ellipse's major-axis radius.
     * @param radiusY The ellipse's minor-axis radius.
     * @param rotation The rotation for this ellipse, expressed in radians
     * @param startAngle The starting point, measured from the x axis, from which it will be drawn, expressed in radians
     * @param endAngle The end ellipse's angle to which it will be drawn, expressed in radians
     */
    public drawEllipse(x: number, y: number, radiusX: number, radiusY: number, rotation: number, startAngle: number, endAngle: number) {
        this._context2d.beginPath();
        this._context2d.ellipse(x, y, radiusX, radiusY, rotation, startAngle, endAngle);
        this._context2d.stroke();
    }
}
